import zmq

from otmcm_backend.common.interfaces.zmq_channel import (
    ZMQReqRepServer
)

from otmcm_backend.common.models.channel_models import (
    OneHostDistributorMangerRequestChannel,
    DistributorMangerUserScriptRequestChannelConfig
)


class ServerDistributorManagerRequestChannel(OneHostDistributorMangerRequestChannel, ZMQReqRepServer):

    CHANNEL_SOCKET_PATTERN = zmq.REP

    def run_after_channel_started_tasks(self):
        # For now do nothing as this is a REP side so it just needs to wait for request and reply
        pass
        

class ServerDistributorMangerUserScriptRequestChannel(DistributorMangerUserScriptRequestChannelConfig, ZMQReqRepServer):

    CHANNEL_SOCKET_PATTERN = zmq.REP

    def generate_channel_address(self) -> str:
        address_template = super().generate_channel_address()
        # This is on server side, so just localhost binding
        return address_template.format(
            host_address="localhost"
        )

    


class DistributorRequestChannel(ZMQReqRepServer):
    pass



import zmq

from otmcm_backend.distributor_manager.models import (
    ServerDistributorManagerRequestChannel,
    ServerDistributorMangerUserScriptRequestChannel
)

class DistributorManager:
    def __init__(self) -> None:
        self._init_internal_attributes()

    def _init_internal_attributes(self):
        self.context = zmq.Context()
        # Be careful when changing the init order of these channels
        self.userscript_req_channel = ServerDistributorMangerUserScriptRequestChannel(context=self.context)
        self.distributor_req_channel = ServerDistributorManagerRequestChannel(context=self.context)